%ex_basic  How to use the h2m functions on the three basic model types:
%          1. Fully connected HMM 2. Left-right HMM 3. Mixture model
%
%	Beware : This is not a real demo, so you should take a look at the
%       source file before executing it if you want to understand what's
%       going on! If using octave --traditional, you should also add the
%       subdirectory h2m/octave to your loadpath using the path command.

% H2M Toolbox, $Revision: 2.3 $        
% O. Cappe, 24/06/96 - 14/08/2001
% http://www.tsi.enst.fr/~cappe/h2m/

% The notations are essentially those of
%   AUTHOR   = "L. R. Rabiner",
%   TITLE    = "A tutorial on hidden Markov models and selected applications
%              in speech recognition",
%   JOURNAL  = procieee,
%   VOLUME   = 77,
%   NUMBER   = 2,
%   MONTH    = feb,
%   PAGES    = "257-285",
%   YEAR     = 1989
%
% pi0:	 Vector of initial probabilities, pi_0(i) = P(s_1 = i)
% A:	 Transition matrix a(i,j) = P(s_{t+1}=j | s_{t}=i)
% mu:    Mean vector of gaussian densities (stacked one above the other)
% Sigma: Covariance matrices (or diagonal of) stacked one above the other
%
% See the on-line help of the functions for more information.

clear;

% Constant: Flags for diagonal and full covariance matrices
FULL_COV = 0;
DIAG_COV = 1;

% Example 1: Ergodic model, with full covariance matrices
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Data generation
p = 2;
N = 3;
T = 100;
n_it = 10;
pi0_sim = rand(1,N); pi0_sim = pi0_sim/ sum(pi0_sim);
A_sim = rand(N); A_sim= A_sim ./ (sum(A_sim')' * ones(1,N));
mu_sim = [1 0; 0 1; -1 0];
Sigma_sim = [0.1 0; 0 0.1; 0.1 -0.03; -0.03 0.1; 0.2 0; 0 0.2];
X = hmm_gen(A_sim, pi0_sim, mu_sim, Sigma_sim, T);
% Initialization values
pi0 = pi0_sim;
A = A_sim;
mu = [0.2 0; 0 0; -0.2 0];
Sigma = 0.2*[diag(ones(1,p)); diag(ones(1,p)); diag(ones(1,p))];
% EM iterations
for i = 1:n_it
  [alpha, beta, logl_tmp, dens] = hmm_fb(X, A, pi0, mu, Sigma);
  logl(i) = logl_tmp;
  [A, pi0] = hmm_tran(alpha, beta, dens, A, pi0);
  [mu, Sigma] = hmm_dens(X, alpha, beta, FULL_COV);
end
% You could use more simply
% [A, pi0, mu, Sigma, logl] = hmm(X, A, pi0, mu, Sigma, n_it);
% Plot estimated means and covariances
clf;
gauselps(mu, Sigma);
hold on;
gauselps(mu_sim, Sigma_sim, 'r');

pause;

% Example 2: Left-right model, with diagonal covariances
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Data generation
p = 2;
N = 3;
T = 30;
K = 10;
n_it = 10;
pi0_sim = sparse([1 0 0]);
A_sim = sparse([0.8 0.16 0.04; 0 0.9 0.1; 0 0 1]);
mu_sim = [1 0; 0 1; -1 0];
Sigma_sim = [0.1 0.1; 0.1 0.2; 0.2 0.2];
X = zeros(T*K,p);
for i = 1:K
  X((1+(i-1)*T:i*T),:) = hmm_gen(A_sim, pi0_sim, mu_sim, Sigma_sim, T);
end
st = 1 + (0:(K-1))*T;
% Initialization values
A = sparse([0.4 0.4 0.2; 0 0.5 0.5; 0 0 1]);;
[mu, Sigma] = hmm_mint(X, st, N, DIAG_COV);
% EM iterations
for i = 1:n_it
  [A, logl(i), gamma] = hmm_mest(X, st, A, mu, Sigma);
  [mu, Sigma] = mix_par(X, gamma, DIAG_COV);
end
% You could use more simply
% [A, mu, Sigma, logl] = lrhmm(X, st, A, mu, Sigma, n_it);
% Plot estimated means and covariances
clf;
gauselps(mu, Sigma);
hold on;
gauselps(mu_sim, Sigma_sim, 'r');

pause;

% Example 3: Gaussian Mixture Model
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Data generation
p = 2;
N = 4;
T = 500;
n_it = 10;
w_sim = [0.2 0.4 0.2 0.2];
mu_sim = [1 0; 0 1; -1 0; 0 -1];
Sigma_sim = [0.1 0; 0 0.1; 0.1 -0.03; -0.03 0.1; 0.2 0; 0 0.2; 0.1 0; 0 0.1];
X = mix_gen(w_sim, mu_sim, Sigma_sim, T);
% Initial values (using svq)
w = ones(1,N)/N;
[mu, label] = svq(X,N,10);
Sigma = [cov(X((label == 1),:)); cov(X((label == 2),:)); cov(X((label == 3),:)); cov(X((label == 4),:))];
% EM iterations
for i = 1:n_it
  [gamma, logl(i)] = mix_post(X, w, mu, Sigma);
  [mu, Sigma, w] = mix_par(X, gamma, FULL_COV);
end
% You could use more simply
% [w, mu, Sigma, logl] = mix(X, w, mu, Sigma, n_it);
% Plot estimated means and covariances
clf;
gauselps(mu, Sigma);
hold on;
gauselps(mu_sim, Sigma_sim, 'r');
